package org.ananas.mac.menu;

/*
 * Copyright 2003, Benoit Marchal. All rights reserved.
 * Distributed under the artistic license (open source).
 * See www.ananas.org for licensing information.
 */

import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;

abstract class MacItem
   implements ApplicationItem
{
   private String label = null;
   private boolean registered = false;
   private ArrayList listeners = new ArrayList();

   public void register(String label,Menu menu)
      throws IllegalStateException
   {
      registerHelper(label);
   }
   
   public void register(String label,MenuShortcut shortcut,Menu menu)
      throws IllegalStateException
   {
      registerHelper(label);
   }
   
   public void register(String label,JMenu menu)
      throws IllegalStateException
   {
      registerHelper(label);
   }

   public void register(Icon icon,JMenu menu)
      throws IllegalStateException
   {
      registerHelper("");
   }
   
   public void register(String label,Icon icon,JMenu menu)
      throws IllegalStateException
   {
      registerHelper(label);
   }
   
   public void register(String label,int mnemonic,JMenu menu)
      throws IllegalStateException
   {
      registerHelper(label);
   }

   public void register(String label,KeyStroke accelerator,JMenu menu)
      throws IllegalStateException
   {
      registerHelper(label);
   }
   
   public void register(Icon icon,KeyStroke accelerator,JMenu menu)
      throws IllegalStateException
   {
      registerHelper("");
   }
   
   public void register(String label,Icon icon,KeyStroke accelerator,JMenu menu)
      throws IllegalStateException
   {
      registerHelper(label);
   }
   
   public void register(String label,int mnemonic,KeyStroke accelerator,JMenu menu)
      throws IllegalStateException
   {
      registerHelper(label);
   }

   public synchronized void addActionListener(ActionListener listener)
      throws IllegalStateException
   {
      if(registered)
         listeners.add(listener);
      else
         throw new IllegalStateException(ApplicationItemFactory.resources.getString("not.registered"));
   }
   
   public synchronized void removeActionListener(ActionListener listener)
      throws IllegalStateException
   {
      if(registered)
      {
         int index = listeners.indexOf(listener);
         if(index != -1)
            listeners.remove(index);
      }
      else
         throw new IllegalStateException(ApplicationItemFactory.resources.getString("not.registered"));
   }

   public boolean isAddedToMenu()
   {
      return false;
   }

   protected synchronized void fireActionPerformed()
   {
      ActionEvent event = new ActionEvent(this,ActionEvent.ACTION_PERFORMED,label);
      Iterator iterator = listeners.iterator();
      while(iterator.hasNext())
      {
         ActionListener listener = (ActionListener)iterator.next();
         try
         {
            listener.actionPerformed(event);
         }
         catch(Exception x)
         {
            x.printStackTrace();
         }
      }
   }
  
   protected abstract void register();
  
   private void registerHelper(String label)
      throws IllegalStateException
   {
      if(registered)
         throw new IllegalStateException(ApplicationItemFactory.resources.getString("registered"));
      else
      {
         registered = true;
         this.label = label;
         register();
      }
   }
}